﻿using Microsoft.DwayneNeed.Utilities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media.Imaging;
using Microsoft.DwayneNeed.Numerics;

namespace RcpServer
{
    public class BitmapBuffer
    {
        public BitmapBuffer()
        {
        }

        public BitmapBuffer(int width, int height)
        {
            Alloc(width, height);
        }

        public BitmapBuffer(Buffer2DView<uint> buffer) : this(buffer.Width, buffer.Height)
        {
            _buffer.SetBits(buffer, new Int32Rect(0, 0, _buffer.Width, _buffer.Height));
        }

        public Buffer2DView<uint> View { get { return _buffer; } }
        public int Width { get { return _buffer.Width; } }
        public int Height { get { return _buffer.Height; } }

        internal void Alloc(int width, int height)
        {
            if (_handle.IsAllocated)
            {
                //TODO: need ref counting for existing views over this, or something...
                _handle.Free();
            }

            _pixels = new uint[width * (height+1)]; // (height+1) is a HACK to work around the over-read due to stride
            _handle = GCHandle.Alloc(_pixels, GCHandleType.Pinned);
            _buffer = new Buffer2D<uint>(_handle.AddrOfPinnedObject(), width, new Int32Rect(0, 0, width, height));
        }

        internal void SetBitsInternal(Buffer2DView<uint> srcView, Int32Rect srcRect, Int32Point? dstPoint = null)
        {
            _buffer.SetBits(srcView, srcRect, dstPoint);
        }

        internal void XorBitsInternal(Buffer2DView<uint> srcView, Int32Rect srcRect, Int32Point? dstPoint = null)
        {
            _buffer.XorBits(srcView, srcRect, dstPoint);
        }

        private uint[] _pixels;
        private GCHandle _handle;
        private Buffer2D<uint> _buffer;
    }
}
