﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Threading;
using Microsoft.DwayneNeed;
using System.Diagnostics;
using System.Runtime.InteropServices;
using Microsoft.DwayneNeed.Utilities;
using Microsoft.DwayneNeed.Win32.User32;
using Microsoft.DwayneNeed.Win32;
using Microsoft.DwayneNeed.Win32.Gdi32;
using Microsoft.DwayneNeed.Win32.Kernel32;

namespace RcpServer
{
    public class ScreenScraperBitmapBuffer : BitmapBuffer
    {
        public ScreenScraperBitmapBuffer(int tileDesiredWidth, int tileDesiredHeight)
        {
            _tileDesiredWidth = tileDesiredWidth;
            _tileDesiredHeight = tileDesiredHeight;

            _timer = new DispatcherTimer(DispatcherPriority.Normal);
            _timer.Interval = TimeSpan.FromMilliseconds(30);
            _timer.Tick += OnTimerTick;
            _timer.Start();
        }

        private void OnTimerTick(object sender, EventArgs e)
        {
            RECT rcClient = new RECT();
            HWND hwnd = NativeMethods.GetDesktopWindow();
            NativeMethods.GetClientRect(hwnd, ref rcClient);

            bool isFullUpdate = false;
            if (_hDC == IntPtr.Zero || rcClient.width != _dibSectionBuffer.Width || rcClient.height != _dibSectionBuffer.Height)
            {
                // Allocate a new bitmap to copy the screen into.
                DestroyBitmap();
                CreateBitmap(rcClient.width, rcClient.height);
                isFullUpdate = true;
            }

            IntPtr hdcSrc = NativeMethods.GetDC(hwnd);
            NativeMethods.BitBlt(_hDC, 0, 0, rcClient.width, rcClient.height, hdcSrc, 0, 0, ROP.SRCCOPY);
            NativeMethods.ReleaseDC(hwnd, hdcSrc);

            HDC hdc = new HDC(_hDC);
            RECT rc = new RECT();
            rc.left = 100;
            rc.top = 100;
            rc.right = 300;
            rc.bottom = 300;
            HBRUSH hbrRed = new HBRUSH(new COLORREF(0xFF, 0xFF, 0));
            //HBRUSH hbrOld = hdc.SelectBrush(hbrRed);
            hdc.FillRect(ref rc, hbrRed);
            //hdc.SelectBrush(hbrOld);
            hbrRed.Dispose();

            //IntPtr hFont = NativeMethods.CreateFont(96, 0, 0, 0, FW.DONTCARE, false, false, false, CHARSET.DEFAULT, OUTPRECIS.OUTLINE, CLIP.DEFAULT_PRECIS, QUALITY.CLEARTYPE, PITCH.VARIABLE, "Impact");
            //IntPtr oldFont = NativeMethods.SelectObject(_hDC, hFont);
            //NativeMethods.SetBkMode(_hDC, BKMODE.TRANSPARENT);
            //NativeMethods.SetBkColor(_hDC, NativeMacros.RGB(0, 0, 0xFF));
            //NativeMethods.SetTextColor(_hDC, NativeMacros.RGB(0, 0xFF, 0));
            //NativeMethods.DrawText(_hDC, "test", -1, ref rcClient, DT.LEFT);
            //NativeMethods.SelectObject(_hDC, oldFont);
            //NativeMethods.DeleteObject(hFont);

            unsafe
            {
                if (isFullUpdate)
                {
                    // For the full update, copy the screen pixels directly
                    // into our buffer.
                    Int32Rect fullRect = new Int32Rect(0,0,rcClient.width, rcClient.height);
                    SetBitsInternal(_dibSectionBuffer, fullRect);

                    // Raise the resized event
                    EventHandler<EventArgs> handlerResized = Resized;
                    if (handlerResized != null)
                    {
                        handlerResized(this, EventArgs.Empty);
                    }

                    // Raise the updated event
                    EventHandler<BitmapBufferUpdatedEventArgs> handlerUpdated = Updated;
                    if (handlerUpdated != null)
                    {
                        handlerUpdated.Invoke(this, new BitmapBufferUpdatedEventArgs(this, fullRect, true));
                    }
                }
                else
                {
                    var queryDirtyRects = from tileRect in GenerateTiles().AsParallel()
                                          where !View.CompareBits(_dibSectionBuffer, tileRect)
                                          select tileRect;
                    List<Int32Rect> dirtyRects = queryDirtyRects.ToList();
                    for (int i = 0; i < dirtyRects.Count; i++)
                    {
                        Int32Rect dirtyRect = dirtyRects[i];

                        // Copy the updates into the our buffer.
                        SetBitsInternal(_dibSectionBuffer, dirtyRect);

                        // Raise the updated event
                        BitmapBufferUpdatedEventArgs updatedArgs = new BitmapBufferUpdatedEventArgs(this, dirtyRect, i == dirtyRects.Count - 1);
                        EventHandler<BitmapBufferUpdatedEventArgs> handlerUpdated = Updated;
                        if (handlerUpdated != null)
                        {
                            handlerUpdated.Invoke(this, updatedArgs);
                        }
                    }
                }
            }
        }

        public event EventHandler<EventArgs> Resized;
        public event EventHandler<BitmapBufferUpdatedEventArgs> Updated;

        // return a list of tiles that cover the current buffer.
        private IEnumerable<Int32Rect> GenerateTiles()
        {
            // Since the enumeration can be lazily extracted, and the
            // underlying buffer can change, snap the current values
            // of width and height.
            int height = _dibSectionBuffer.Height;
            int width = _dibSectionBuffer.Width;

            for (int y = 0; y < height; y += _tileDesiredHeight)
            {
                int tileHeight = _tileDesiredHeight;
                if (y + tileHeight > height)
                {
                    tileHeight = height - y;
                }

                for (int x = 0; x < width; x += _tileDesiredWidth)
                {
                    int tileWidth = _tileDesiredWidth;
                    if (x + tileWidth > width)
                    {
                        tileWidth = width - x;
                    }

                    yield return new Int32Rect(x, y, tileWidth, tileHeight);
                }
            }
        }

        private void CreateBitmap(int width, int height)
        {
            Debug.Assert(_hSection == IntPtr.Zero);
            Debug.Assert(_hBitmap == IntPtr.Zero);

            if (width == 0 || height == 0) return;

            int size = height * width * 4;

            _hSection = NativeMethods.CreateFileMapping(
                PAGE.READWRITE,
                SEC.NONE,
                0,
                (uint)size,
                null);

            BITMAPINFO bmi = new BITMAPINFO();
            bmi.biSize = Marshal.SizeOf(bmi);
            bmi.biWidth = width;
            bmi.biHeight = -height; // top-down
            bmi.biPlanes = 1;
            bmi.biBitCount = 32;
            bmi.biCompression = BI.RGB;

            IntPtr hdcScreen = NativeMethods.GetDC(HWND.NULL);

            IntPtr pBits = IntPtr.Zero;
            _hBitmap = NativeMethods.CreateDIBSection(
                hdcScreen,
                ref bmi,
                DIB.RGB_COLORS,
                out pBits,
                _hSection,
                0);

            // TODO: probably don't need a new DC every time...
            _hDC = NativeMethods.CreateCompatibleDC(hdcScreen);
            NativeMethods.SelectObject(_hDC, _hBitmap);

            _dibSectionBuffer = new Buffer2D<uint>(pBits, width, new Int32Rect(0, 0, width, height));
            Alloc(width, height);
        }

        private void DestroyBitmap()
        {
            if (_hDC != IntPtr.Zero)
            {
                NativeMethods.DeleteObject(_hDC);
                _hDC = IntPtr.Zero;
            }

            if (_hBitmap != IntPtr.Zero)
            {
                NativeMethods.DeleteObject(_hBitmap);
                _hBitmap = IntPtr.Zero;
            }

            _hDC = IntPtr.Zero;

            if (_hSection != IntPtr.Zero)
            {
                NativeMethods.CloseHandle(_hSection);
                _hSection = IntPtr.Zero;
            }

            _dibSectionBuffer = new Buffer2D<uint>();
        }

        private int _tileDesiredWidth;
        private int _tileDesiredHeight;
        private DispatcherTimer _timer;

        private IntPtr _hSection;
        private IntPtr _hBitmap;
        private IntPtr _hDC;
        private Buffer2D<uint> _dibSectionBuffer;
    }
}
