var PerfMonitor = (function () {
    function PerfMonitor(element) {
        this.element = element;
        this.divperfcounters = $(element).find("#perfcounters");
        this.divperfgraph = $(element).find("#perfgraph");
        this.graph = new PerfGraph(this.divperfgraph.find("#graph"));
        this.divperfdetails = $(element).find("#perfdetails");
    }
    PerfMonitor.prototype.Reset = function () {
        this.graph.InitGraph();
    };
    PerfMonitor.prototype.start = function () {
        var _this = this;
        this.timerToken = setInterval(function () {
            return _this.monitor();
        }, 1000);
    };
    PerfMonitor.prototype.stop = function () {
        clearTimeout(this.timerToken);
    };
    PerfMonitor.prototype.monitor = function () {
        var items = _perfStore.GetItems();
        if(items.length == 0) {
            return;
        }
        var datavalues = new Array();
        for(var i = 0; i < items.length; i++) {
            var diff = (Date.now() - items[i].Start()) / 1000;
            var id = "pci_" + items[i].Id();
            var perfcounteritem = this.divperfcounters.find("#" + id);
            if(perfcounteritem.size() == 0) {
                perfcounteritem = this.createPerfItemSet(id, items[i]);
                this.divperfcounters.append(perfcounteritem);
            }
            perfcounteritem.find("#name").text(items[i].Id());
            perfcounteritem.find("#count").text(items[i].Count());
            perfcounteritem.find("#best").text(items[i].Best().toFixed(3));
            perfcounteritem.find("#avg").text(items[i].Avg().toFixed(3));
            perfcounteritem.find("#worst").text(items[i].Worst().toFixed(3));
            perfcounteritem.find("#cps").text((items[i].Count() / diff).toFixed(3));
            perfcounteritem.find("#aps").text(((items[i].AvgAmount() / diff) * items[i].Count()).toFixed(3));
            var graphval = ((items[i].AvgAmount() / diff) * items[i].Count());
            datavalues.push(graphval);
        }
        this.graph.AddPointToGraph(datavalues);
    };
    PerfMonitor.prototype.createPerfItemSet = function (id, itemc) {
        var perfcounteritem = $("<div id=" + id + " class='perfitem'/>");
        perfcounteritem.append($("<div id='name'/>"));
        perfcounteritem.append($("<div id='count'/>"));
        perfcounteritem.append($("<div id='best'/>"));
        perfcounteritem.append($("<div id='avg'/>"));
        perfcounteritem.append($("<div id='worst'/>"));
        perfcounteritem.append($("<div id='cps'/>"));
        perfcounteritem.append($("<div id='aps'/>"));
        return perfcounteritem;
    };
    return PerfMonitor;
})();
var GraphLine = (function () {
    function GraphLine() { }
    return GraphLine;
})();
var PerfGraph = (function () {
    function PerfGraph(element) {
        this.graph = element;
        this.xPadding = 5;
        this.yPadding = 5;
        this.colors = [
            "#F00", 
            "#AAA", 
            "#DDD", 
            "#CCC"
        ];
        this.lines = new Array();
        this.InitGraph();
    }
    PerfGraph.prototype.getMaxY = function () {
        var max = 100;
        max += 10 - max % 10;
        return max;
    };
    PerfGraph.prototype.getXPixel = function (val) {
        return ((this.graph.width() - this.xPadding) / 100) * val + (this.xPadding * 1.5);
    };
    PerfGraph.prototype.getYPixel = function (val) {
        return this.graph.height() - (((this.graph.height() - this.yPadding) / this.getMaxY()) * val) - this.yPadding;
    };
    PerfGraph.prototype.InitGraph = function () {
        this.graph[0].width = this.graph[0].width;
        this.timeX = 0;
        var c = this.graph[0].getContext('2d');
        c.lineWidth = 1;
        c.strokeStyle = '#333';
        c.font = 'italic 8pt sans-serif';
        c.textAlign = "center";
        c.beginPath();
        c.moveTo(this.xPadding, 0);
        c.lineTo(this.xPadding, this.graph.height() - this.yPadding);
        c.lineTo(this.graph.width(), this.graph.height() - this.yPadding);
        c.stroke();
        c.textAlign = "right";
        c.textBaseline = "middle";
        c.beginPath();
        c.moveTo(this.getXPixel(0), this.getYPixel(0));
    };
    PerfGraph.prototype.AddPointToGraph = function (datavalue) {
        if(this.timeX == 100) {
            this.InitGraph();
        }
        var c = this.graph[0].getContext('2d');
        c.font = 'italic 4pt sans-serif';
        for(var i = 0; i < datavalue.length; i++) {
            var newline;
            if(this.lines.length <= i) {
                var newline = new GraphLine();
                newline.color = this.colors[i];
                newline.X = this.timeX;
                newline.Y = 0;
                this.lines.push(newline);
            } else {
                newline = this.lines[i];
            }
            c.strokeStyle = '#f00';
            c.strokeStyle = this.colors[i];
            c.moveTo(this.getXPixel(this.timeX - 1), this.getYPixel(newline.Y));
            c.lineTo(this.getXPixel(this.timeX), this.getYPixel(datavalue[i]));
            newline.Y = datavalue[i];
            c.stroke();
            c.fillStyle = '#333';
            c.beginPath();
            c.arc(this.getXPixel(this.timeX), this.getYPixel(datavalue[i]), 0, 0, Math.PI * 2, true);
            c.fill();
        }
        this.timeX++;
    };
    return PerfGraph;
})();
