module Telemetry {
    export class PerfStore{
        private static uniqueInstance : PerfStore;
        public static getInstance(): PerfStore {
            if (uniqueInstance == null) {
                uniqueInstance = new PerfStore();
            } 
            return uniqueInstance;
        }
        constructor () { 
            this._items = new PerfStoreItemCollection[];
        }
        public Reset() { 
            this._items = new PerfStoreItemCollection[];
        }
        private _items: PerfStoreItemCollection[];
        public Start(id) { 
            //check for null
            var item = new PerfStoreItem();
            item.start = Date.now();
            var itemc = this.getCollection(id);
            if (itemc == null)
            {
                itemc = new PerfStoreItemCollection(id);
                this._items.push(itemc);
            }
            return itemc.Add(item);
        }
        public End(id:string, key:number, amount:number) { 
            var end = Date.now();
            var itemc = this.getCollection(id);
            if (itemc == null)
                return;
            itemc.UpdateItem(end, key, amount);
        }
        private getCollection(id:string) { 
            for (var i = 0; i < this._items.length; i++)
            { 
                if (this._items[i].Id() == id)
                    return this._items[i];
            }
            return null;
        }
        public GetItems() { 
            return this._items;
        }

    }

    export class PerfStoreItemCollection {
        constructor(id: string) {
            this._items = new PerfStoreItem[];
            this._id = id;
            this._avg = 0;
            this._best = 0;
            this._count = 0;
            this._worst = 0;
            this._avgAmount = 0;
            this._start = Date.now();
        }
        private _items: PerfStoreItem[];
        private _id: string;
        private _best: number;
        private _avg: number;
        private _worst: number;
        private _count: number;
        private _avgAmount: number;
        private _start: number;
        public Id() { return this._id; }
        public Best() { return this._best; }
        public Avg() { return this._avg; }
        public Worst() { return this._worst; }
        public Count() { return this._count; }
        public AvgAmount() { return this._avgAmount; }
        public Start() { return this._start; }
        public UpdateItem(end:number, key:number, amount:number) { 
            if (this._items.length > 0)
            {
                var item = this.getItem(key);
                if (item == null)
                    return;
                item.end = end;
                item.amount = amount;
                //this._items[this._items.length - 1] = item;
                var diff = item.end - item.start;
                this._count++;
                if (diff < this._best || this._best == 0)
                    this._best = diff;
                if (diff > this._worst)
                    this._worst = diff;
                this._avg = ((diff - this._avg) / this._count) + this._avg;
                this._avgAmount = ((amount - this._avgAmount) / this._count) + this._avgAmount;
            }
        }
       
        public GetItems()
        {
            return this._items;
        }
        public GetItem(i:number) { 
            return this._items[i];
        }

        private getItem(key: number) { 
            for (var i = this._items.length - 1; i > -1; i--) {
                if (this._items[i].key == key) {
                    return this._items[i];
                }
            }
        return null;
        }
        public Add(item: PerfStoreItem) { 
            //TODO: lock object
            item.key = this._items.length;
            this._items.push(item);
            return item.key;
            
        }
    }
    export class PerfStoreItem {
        public start: number;
        public end: number;
        public amount: number;
        public key: number;
    }

}
